@@ -22,7 +22,6 @@
 #include "nm-act-request.h"
 #include "nm-l3-config-data.h"
 #include "nm-manager.h"
-#include "nm-manager.h"
 #include "nm-setting-wifi-p2p.h"
 #include "nm-utils.h"
 #include "nm-wifi-p2p-peer.h"
@@ -1229,8 +1228,6 @@ nm_device_wifi_p2p_new(const char *iface)
                         NM_DEVICE_TYPE_WIFI_P2P,
                         NM_DEVICE_LINK_TYPE,
                         NM_LINK_TYPE_WIFI,
-                        NM_DEVICE_RFKILL_TYPE,
-                        RFKILL_TYPE_WLAN,
                         NULL);
 }
 
@@ -1293,6 +1290,8 @@ nm_device_wifi_p2p_class_init(NMDeviceWifiP2PClass *klass)
 
     device_class->state_changed = device_state_changed;
 
+    device_class->rfkill_type = NM_RFKILL_TYPE_WLAN;
+
     obj_properties[PROP_PEERS] = g_param_spec_boxed(NM_DEVICE_WIFI_P2P_PEERS,
                                                     "",
                                                     "",
