@@ -136,6 +136,11 @@ nm_vpn_service_plugin_set_state(NMVpnServicePlugin *plugin, NMVpnServiceState st
     }
 }
 
+/**
+ * nm_vpn_service_plugin_set_login_banner:
+ *
+ * Since: 1.2
+ */
 void
 nm_vpn_service_plugin_set_login_banner(NMVpnServicePlugin *plugin, const char *banner)
 {
@@ -160,6 +165,11 @@ _emit_failure(NMVpnServicePlugin *plugin, NMVpnPluginFailure reason)
         nmdbus_vpn_plugin_emit_failure(priv->dbus_vpn_service_plugin, reason);
 }
 
+/**
+ * nm_vpn_service_plugin_failure:
+ *
+ * Since: 1.2
+ */
 void
 nm_vpn_service_plugin_failure(NMVpnServicePlugin *plugin, NMVpnPluginFailure reason)
 {
@@ -169,6 +179,11 @@ nm_vpn_service_plugin_failure(NMVpnServicePlugin *plugin, NMVpnPluginFailure rea
     nm_vpn_service_plugin_disconnect(plugin, NULL);
 }
 
+/**
+ * nm_vpn_service_plugin_disconnect:
+ *
+ * Since: 1.2
+ */
 gboolean
 nm_vpn_service_plugin_disconnect(NMVpnServicePlugin *plugin, GError **err)
 {
@@ -326,6 +341,11 @@ schedule_fail_stop(NMVpnServicePlugin *plugin, guint timeout_secs)
         priv->fail_stop_id = g_idle_add(fail_stop, plugin);
 }
 
+/**
+ * nm_vpn_service_plugin_set_config:
+ *
+ * Since: 1.2
+ */
 void
 nm_vpn_service_plugin_set_config(NMVpnServicePlugin *plugin, GVariant *config)
 {
@@ -364,6 +384,11 @@ nm_vpn_service_plugin_set_config(NMVpnServicePlugin *plugin, GVariant *config)
         nm_vpn_service_plugin_set_state(plugin, NM_VPN_SERVICE_STATE_STARTED);
 }
 
+/**
+ * nm_vpn_service_plugin_set_ip4_config:
+ *
+ * Since: 1.2
+ */
 void
 nm_vpn_service_plugin_set_ip4_config(NMVpnServicePlugin *plugin, GVariant *ip4_config)
 {
@@ -422,6 +447,11 @@ nm_vpn_service_plugin_set_ip4_config(NMVpnServicePlugin *plugin, GVariant *ip4_c
         nm_vpn_service_plugin_set_state(plugin, NM_VPN_SERVICE_STATE_STARTED);
 }
 
+/**
+ * nm_vpn_service_plugin_set_ip6_config:
+ *
+ * Since: 1.2
+ */
 void
 nm_vpn_service_plugin_set_ip6_config(NMVpnServicePlugin *plugin, GVariant *ip6_config)
 {
@@ -729,54 +759,6 @@ nm_vpn_service_plugin_secrets_required(NMVpnServicePlugin *plugin,
 
 /*****************************************************************************/
 
-typedef struct {
-    char *buf;
-    gsize n_buf;
-    int   fd;
-    bool  eof : 1;
-    char  buf_full[1024];
-} ReadFdBuf;
-
-static inline gboolean
-_read_fd_buf_c(ReadFdBuf *read_buf, char *ch)
-{
-    gssize n_read;
-
-    if (read_buf->n_buf > 0)
-        goto out_data;
-    if (read_buf->eof)
-        return FALSE;
-
-again:
-    n_read = read(read_buf->fd, read_buf->buf_full, sizeof(read_buf->buf_full));
-    if (n_read <= 0) {
-        if (n_read < 0 && errno == EAGAIN) {
-            struct pollfd pfd;
-            int           r;
-
-            memset(&pfd, 0, sizeof(pfd));
-            pfd.fd     = read_buf->fd;
-            pfd.events = POLLIN;
-
-            r = poll(&pfd, 1, -1);
-            if (r > 0)
-                goto again;
-            /* error or timeout. Fall through and set EOF. */
-        }
-        read_buf->eof = TRUE;
-        return FALSE;
-    }
-
-    read_buf->buf   = read_buf->buf_full;
-    read_buf->n_buf = n_read;
-
-out_data:
-    read_buf->n_buf--;
-    *ch = read_buf->buf[0];
-    read_buf->buf++;
-    return TRUE;
-}
-
 #define DATA_KEY_TAG   "DATA_KEY="
 #define DATA_VAL_TAG   "DATA_VAL="
 #define SECRET_KEY_TAG "SECRET_KEY="
@@ -808,7 +790,7 @@ nm_vpn_service_plugin_read_vpn_details(int fd, GHashTable **out_data, GHashTable
     nm_auto_free_gstring GString  *val     = NULL;
     nm_auto_free_gstring GString  *line    = NULL;
     GString                       *str     = NULL;
-    ReadFdBuf                      read_buf;
+    char                           c;
 
     if (out_data)
         g_return_val_if_fail(*out_data == NULL, FALSE);
@@ -819,27 +801,37 @@ nm_vpn_service_plugin_read_vpn_details(int fd, GHashTable **out_data, GHashTable
     secrets =
         g_hash_table_new_full(nm_str_hash, g_str_equal, g_free, (GDestroyNotify) nm_free_secret);
 
-    read_buf.buf   = NULL;
-    read_buf.n_buf = 0;
-    read_buf.fd    = fd;
-    read_buf.eof   = FALSE;
-
     line = g_string_new(NULL);
 
     /* Read stdin for data and secret items until we get a DONE */
     while (1) {
-        gboolean eof;
-        char     c = '\0';
+        ssize_t nr;
 
-        eof = !_read_fd_buf_c(&read_buf, &c);
+        nr = read(fd, &c, 1);
+        if (nr < 0) {
+            if (errno == EAGAIN) {
+                struct pollfd pfd;
+                int           r;
 
-        if (!eof && c == '\0') {
-            /* On the first '\0' char, we also assume the data is finished. Abort. */
-            read_buf.eof = TRUE;
-            eof          = TRUE;
+                memset(&pfd, 0, sizeof(pfd));
+                pfd.fd     = fd;
+                pfd.events = POLLIN;
+
+                r = poll(&pfd, 1, -1);
+                if (r > 0)
+                    continue;
+
+                /* error or timeout. Fall through and break. */
+            }
+            break;
         }
 
-        if (!eof && c != '\n') {
+        if (nr > 0 && c == '\0') {
+            /* '\0' are not supported. Replace with newline. */
+            c = '\n';
+        }
+
+        if (nr > 0 && c != '\n') {
             g_string_append_c(line, c);
             if (line->len > 512 * 1024) {
                 /* we are about to read a huge line. That is not right, abort. */
@@ -893,7 +885,7 @@ nm_vpn_service_plugin_read_vpn_details(int fd, GHashTable **out_data, GHashTable
 next:
         g_string_truncate(line, 0);
 
-        if (eof)
+        if (nr == 0)
             break;
     }
 
