@@ -935,7 +935,7 @@ nm_tc_tfilter_get_parent(NMTCTfilter *tfilter)
  *
  * Returns: the action associated with a traffic filter.
  *
- * Since: 1.12
+ * Since: 1.42
  **/
 NMTCAction *
 nm_tc_tfilter_get_action(NMTCTfilter *tfilter)
@@ -956,7 +956,7 @@ nm_tc_tfilter_get_action(NMTCTfilter *tfilter)
  *
  * Sets the action associated with a traffic filter.
  *
- * Since: 1.12
+ * Since: 1.42
  **/
 void
 nm_tc_tfilter_set_action(NMTCTfilter *tfilter, NMTCAction *action)
@@ -1475,7 +1475,7 @@ next:
 }
 
 static GVariant *
-tc_qdiscs_get(_NM_SETT_INFO_PROP_TO_DBUS_FCN_ARGS _nm_nil)
+qdiscs_to_dbus(_NM_SETT_INFO_PROP_TO_DBUS_FCN_ARGS _nm_nil)
 {
     gs_unref_ptrarray GPtrArray *qdiscs = NULL;
 
@@ -1484,7 +1484,7 @@ tc_qdiscs_get(_NM_SETT_INFO_PROP_TO_DBUS_FCN_ARGS _nm_nil)
 }
 
 static gboolean
-tc_qdiscs_set(_NM_SETT_INFO_PROP_FROM_DBUS_FCN_ARGS _nm_nil)
+qdiscs_from_dbus(_NM_SETT_INFO_PROP_FROM_DBUS_FCN_ARGS _nm_nil)
 {
     gs_unref_ptrarray GPtrArray *qdiscs = NULL;
 
@@ -1661,7 +1661,7 @@ next:
 }
 
 static GVariant *
-tc_tfilters_get(_NM_SETT_INFO_PROP_TO_DBUS_FCN_ARGS _nm_nil)
+tfilters_to_dbus(_NM_SETT_INFO_PROP_TO_DBUS_FCN_ARGS _nm_nil)
 {
     gs_unref_ptrarray GPtrArray *tfilters = NULL;
 
@@ -1670,7 +1670,7 @@ tc_tfilters_get(_NM_SETT_INFO_PROP_TO_DBUS_FCN_ARGS _nm_nil)
 }
 
 static gboolean
-tc_tfilters_set(_NM_SETT_INFO_PROP_FROM_DBUS_FCN_ARGS _nm_nil)
+tfilters_from_dbus(_NM_SETT_INFO_PROP_FROM_DBUS_FCN_ARGS _nm_nil)
 {
     gs_unref_ptrarray GPtrArray *tfilters = NULL;
 
@@ -1944,8 +1944,8 @@ nm_setting_tc_config_class_init(NMSettingTCConfigClass *klass)
      *        <para>
      *     can be used to set a different hash table size, available
      *     from kernel 2.6.39 onwards.  The specified divisor must be
-     *     a power of two and cannot be larger than 65536.  Default
-     *     value: 1024.
+     *     a power of two and cannot be larger than 65536.
+     *     Default value: 1024.
      *        </para>
      *      </listitem>
      *  </varlistentry>
@@ -2083,12 +2083,13 @@ nm_setting_tc_config_class_init(NMSettingTCConfigClass *klass)
                                                      G_TYPE_PTR_ARRAY,
                                                      G_PARAM_READWRITE | NM_SETTING_PARAM_INFERRABLE
                                                          | G_PARAM_STATIC_STRINGS);
-    _nm_properties_override_gobj(properties_override,
-                                 obj_properties[PROP_QDISCS],
-                                 NM_SETT_INFO_PROPERT_TYPE_DBUS(NM_G_VARIANT_TYPE("aa{sv}"),
-                                                                .to_dbus_fcn   = tc_qdiscs_get,
-                                                                .compare_fcn   = compare_fcn_qdiscs,
-                                                                .from_dbus_fcn = tc_qdiscs_set, ));
+    _nm_properties_override_gobj(
+        properties_override,
+        obj_properties[PROP_QDISCS],
+        NM_SETT_INFO_PROPERT_TYPE_DBUS(NM_G_VARIANT_TYPE("aa{sv}"),
+                                       .to_dbus_fcn   = qdiscs_to_dbus,
+                                       .compare_fcn   = compare_fcn_qdiscs,
+                                       .from_dbus_fcn = qdiscs_from_dbus, ));
 
     /**
      * NMSettingTCConfig:tfilters: (type GPtrArray(NMTCTfilter))
@@ -2107,8 +2108,8 @@ nm_setting_tc_config_class_init(NMSettingTCConfigClass *klass)
      * format: GPtrArray(NMTCTfilter)
      * description-docbook:
      *  <para>
-     * Array of TC traffic filters. Traffic control can manage the packet content during
-     * classification by using filters.
+     *   Array of TC traffic filters. Traffic control can manage the packet content during
+     *   classification by using filters.
      *  </para>
      *  <para>
      *   Each tfilters can be specified by the following attributes:
@@ -2206,7 +2207,7 @@ nm_setting_tc_config_class_init(NMSettingTCConfigClass *klass)
      * ---end---
      **/
     /* ---ifcfg-rh---
-     * property: qdiscs
+     * property: tfilters
      * variable: FILTER1(+), FILTER2(+), ..., TC_COMMIT(+)
      * description: Traffic filters to set on the interface. When no
      *  QDISC1, QDISC2, ..., FILTER1, FILTER2, ... keys are present,
@@ -2225,9 +2226,9 @@ nm_setting_tc_config_class_init(NMSettingTCConfigClass *klass)
         properties_override,
         obj_properties[PROP_TFILTERS],
         NM_SETT_INFO_PROPERT_TYPE_DBUS(NM_G_VARIANT_TYPE("aa{sv}"),
-                                       .to_dbus_fcn   = tc_tfilters_get,
+                                       .to_dbus_fcn   = tfilters_to_dbus,
                                        .compare_fcn   = compare_fcn_tfilter,
-                                       .from_dbus_fcn = tc_tfilters_set, ));
+                                       .from_dbus_fcn = tfilters_from_dbus, ));
 
     g_object_class_install_properties(object_class, _PROPERTY_ENUMS_LAST, obj_properties);
 
