@@ -43,6 +43,7 @@
 #include "nm-setting-ip-tunnel.h"
 #include "nm-setting-ip4-config.h"
 #include "nm-setting-ip6-config.h"
+#include "nm-setting-loopback.h"
 #include "nm-setting-macsec.h"
 #include "nm-setting-macvlan.h"
 #include "nm-setting-match.h"
@@ -189,14 +190,14 @@ NM_TERNARY_TO_OPTION_BOOL(NMTernary v)
 
 NMSetting **_nm_connection_get_settings_arr(NMConnection *connection);
 
-typedef enum { /*< skip >*/
-               NM_SETTING_PARSE_FLAGS_NONE        = 0,
-               NM_SETTING_PARSE_FLAGS_STRICT      = 1LL << 0,
-               NM_SETTING_PARSE_FLAGS_BEST_EFFORT = 1LL << 1,
-               NM_SETTING_PARSE_FLAGS_NORMALIZE   = 1LL << 2,
+typedef enum /*< skip >*/ {
+    NM_SETTING_PARSE_FLAGS_NONE        = 0,
+    NM_SETTING_PARSE_FLAGS_STRICT      = 1LL << 0,
+    NM_SETTING_PARSE_FLAGS_BEST_EFFORT = 1LL << 1,
+    NM_SETTING_PARSE_FLAGS_NORMALIZE   = 1LL << 2,
 
-               _NM_SETTING_PARSE_FLAGS_LAST,
-               NM_SETTING_PARSE_FLAGS_ALL = ((_NM_SETTING_PARSE_FLAGS_LAST - 1) << 1) - 1,
+    _NM_SETTING_PARSE_FLAGS_LAST,
+    NM_SETTING_PARSE_FLAGS_ALL = ((_NM_SETTING_PARSE_FLAGS_LAST - 1) << 1) - 1,
 } NMSettingParseFlags;
 
 gboolean _nm_connection_replace_settings(NMConnection       *connection,
@@ -367,31 +368,6 @@ void _nm_dbus_errors_init(void);
 
 extern gboolean _nm_utils_is_manager_process;
 
-gboolean
-_nm_dbus_typecheck_response(GVariant *response, const GVariantType *reply_type, GError **error);
-
-gulong _nm_dbus_signal_connect_data(GDBusProxy         *proxy,
-                                    const char         *signal_name,
-                                    const GVariantType *signature,
-                                    GCallback           c_handler,
-                                    gpointer            data,
-                                    GClosureNotify      destroy_data,
-                                    GConnectFlags       connect_flags);
-#define _nm_dbus_signal_connect(proxy, name, signature, handler, data) \
-    _nm_dbus_signal_connect_data(proxy, name, signature, handler, data, NULL, (GConnectFlags) 0)
-
-GVariant *_nm_dbus_proxy_call_finish(GDBusProxy         *proxy,
-                                     GAsyncResult       *res,
-                                     const GVariantType *reply_type,
-                                     GError            **error);
-
-GVariant *_nm_dbus_connection_call_finish(GDBusConnection    *dbus_connection,
-                                          GAsyncResult       *result,
-                                          const GVariantType *reply_type,
-                                          GError            **error);
-
-gboolean _nm_dbus_error_has_name(GError *error, const char *dbus_error_name);
-
 /*****************************************************************************/
 
 char *_nm_utils_ssid_to_utf8(GBytes *ssid);
@@ -419,6 +395,7 @@ GSList *_nm_vpn_plugin_info_list_load_dir(const char               *dirname,
 /*****************************************************************************/
 
 GHashTable *_nm_setting_ovs_external_ids_get_data(NMSettingOvsExternalIDs *self);
+GHashTable *_nm_setting_ovs_other_config_get_data(NMSettingOvsOtherConfig *self);
 
 /*****************************************************************************/
 
@@ -481,6 +458,14 @@ NMSettingIPConfig *nm_connection_get_setting_ip_config(NMConnection *connection,
 
 /*****************************************************************************/
 
+struct _NMRefString;
+
+void _nm_connection_set_path_rstr(NMConnection *connection, struct _NMRefString *path);
+
+struct _NMRefString *_nm_connection_get_path_rstr(NMConnection *connection);
+
+/*****************************************************************************/
+
 typedef enum {
     NM_BOND_OPTION_TYPE_INT,
     NM_BOND_OPTION_TYPE_BOTH,
@@ -491,8 +476,6 @@ typedef enum {
 
 NMBondOptionType _nm_setting_bond_get_option_type(NMSettingBond *setting, const char *name);
 
-const char *nm_setting_bond_get_option_or_default(NMSettingBond *self, const char *option);
-
 #define NM_BOND_AD_ACTOR_SYSTEM_DEFAULT "00:00:00:00:00:00"
 
 /*****************************************************************************/
@@ -519,6 +502,11 @@ NMConnectionMultiConnect _nm_connection_get_multi_connect(NMConnection *connecti
 
 gboolean _nm_setting_bond_option_supported(const char *option, NMBondMode mode);
 
+guint8  _nm_setting_bond_opt_value_as_u8(NMSettingBond *s_bond, const char *opt);
+guint16 _nm_setting_bond_opt_value_as_u16(NMSettingBond *s_bond, const char *opt);
+guint32 _nm_setting_bond_opt_value_as_u32(NMSettingBond *s_bond, const char *opt);
+bool    _nm_setting_bond_opt_value_as_intbool(NMSettingBond *s_bond, const char *opt);
+
 /*****************************************************************************/
 
 GPtrArray *_nm_setting_bridge_get_vlans(NMSettingBridge *setting);
@@ -579,6 +567,7 @@ gboolean _nm_utils_dhcp_duid_valid(const char *duid, GBytes **out_duid_bin);
 gboolean _nm_setting_sriov_sort_vfs(NMSettingSriov *setting);
 gboolean _nm_setting_bridge_port_sort_vlans(NMSettingBridgePort *setting);
 gboolean _nm_setting_bridge_sort_vlans(NMSettingBridge *setting);
+gboolean _nm_setting_ovs_port_sort_trunks(NMSettingOvsPort *self);
 
 /*****************************************************************************/
 
@@ -815,6 +804,44 @@ struct _NMSettInfoProperty {
      * Set this flag to force always converting the property even if the value
      * is the default. */
     bool to_dbus_including_default : 1;
+
+    /* This indicates, that the property is deprecated (on D-Bus) for another property.
+     * See also _nm_setting_use_legacy_property() how that works.
+     *
+     * The to_dbus_fcn() will be skipped for such properties, if _nm_utils_is_manager_process
+     * is FALSE. */
+    bool to_dbus_only_in_manager_process : 1;
+
+    /* Whether the property is deprecated.
+     *
+     * Note that we have various representations of profiles, e.g. on D-Bus, keyfile,
+     * nmcli, libnm/NMSetting. Usually a property (in the general sense) is named and
+     * applies similarly to all those. But not always, for example, on D-Bus we
+     * have the field "ethernet.assigned-mac-address", but that exists nowhere
+     * as a property in the other parts (the real property is called
+     * "ethernet.cloned-mac-address").
+     *
+     * This flag indicates whether a property is deprecated. Here "property" means
+     * no specific representation. When a property is deprecated this way, it will
+     * also indirectly apply to setting the property on D-Bus, keyfile, nmcli, etc.
+     * It means the general concept of this thing is no longer useful/recommended.
+     */
+    bool is_deprecated : 1;
+
+    /* Whether the property is deprecated in the D-Bus API.
+     *
+     * This has no real effect (for now). It is only self-documenting code that
+     * this property is discouraged on D-Bus. We might honor this when generating
+     * documentation, or we might use it to find properties that are deprecated.
+     *
+     * Note what this means. For example, "802-1x.phase2-subject-match" is deprecated
+     * as a property altogether, but that does not mean it's deprecated specifically on
+     * D-Bus.
+     * "ethernet.cloned-mac-address" is deprecated on D-Bus in favor of
+     * "ethernet.assigned-mac-address", but the property clone-mac-address itself
+     * is not deprecated. This flag is about the deprecation of the D-Bus representation
+     * of a property. */
+    bool dbus_deprecated : 1;
 };
 
 typedef struct {
@@ -1040,4 +1067,8 @@ gboolean _nm_ip_tunnel_mode_is_layer2(NMIPTunnelMode mode);
 
 GPtrArray *_nm_setting_ip_config_get_dns_array(NMSettingIPConfig *setting);
 
+gboolean nm_connection_need_secrets_for_rerequest(NMConnection *connection);
+
+const GPtrArray *_nm_setting_ovs_port_get_trunks_arr(NMSettingOvsPort *self);
+
 #endif
