@@ -26,7 +26,7 @@
  * view. That is mainly, because such objects are themselves tied to an ifindex.
  *
  * However, for certain objects that's not the case. For example, policy routing
- * rules, certain route types (blackhole, unavailable, prohibit, throw) and MPTCP
+ * rules, certain route types (blackhole, unreachable, prohibit, throw) and MPTCP
  * endpoints require a holistic view of the system. That is, because rules and
  * these route types have no ifindex. For MPTCP endpoints, they have an ifindex,
  * however we can only configure a small number of them at a time, so we need a
@@ -198,7 +198,7 @@ _track_data_hash(gconstpointer data)
     _track_data_assert(track_data, FALSE);
 
     nm_hash_init(&h, 269297543u);
-    nmp_object_id_hash_update(track_data->obj, &h);
+    nmp_object_hash_update(track_data->obj, &h);
     nm_hash_update_val(&h, track_data->user_tag);
     return nm_hash_complete(&h);
 }
@@ -213,7 +213,7 @@ _track_data_equal(gconstpointer data_a, gconstpointer data_b)
     _track_data_assert(track_data_b, FALSE);
 
     return track_data_a->user_tag == track_data_b->user_tag
-           && nmp_object_id_equal(track_data_a->obj, track_data_b->obj);
+           && nmp_object_equal(track_data_a->obj, track_data_b->obj);
 }
 
 static void
@@ -253,6 +253,22 @@ _track_obj_data_get_best_data(TrackObjData *obj_data)
         td_best = track_data;
     }
 
+    if (!td_best)
+        return NULL;
+
+    /* Always copy the object from the best TrackData to the TrackObjData. It is
+     * a bit odd that this getter modifies TrackObjData. However, it gives the
+     * nice property that after calling _track_obj_data_get_best_data() you can
+     * use obj_data->obj (and get the same as td_best->obj).
+     *
+     * This is actually important, because the previous obj_data->obj will have
+     * the same ID, but it might have minor differences to td_best->obj.
+     *
+     * Note that at this point obj_data->obj also might be an object that is no longer
+     * tracked. Updating the reference will ensure that we don't have such old references
+     * around and update to use the most appropriate one. */
+    nmp_object_ref_set(&obj_data->obj, td_best->obj);
+
     return td_best;
 }
 
@@ -784,7 +800,7 @@ nmp_global_tracker_sync_mptcp_addrs(NMPGlobalTracker *self, gboolean reapply)
 
         /* Now, drop all duplicates addresses. Only keep the first one. */
         for (i = 0, j = 0; i < entries->len; i++) {
-            const MptcpSyncData       *d          = nm_g_array_index_p(entries, MptcpSyncData, i);
+            const MptcpSyncData       *d          = &nm_g_array_index(entries, MptcpSyncData, i);
             const NMPlatformMptcpAddr *mptcp_addr = NMP_OBJECT_CAST_MPTCP_ADDR(d->obj_data->obj);
 
             obj_data = g_hash_table_lookup(entries_hash_by_addr, (gpointer) mptcp_addr);
@@ -810,7 +826,7 @@ nmp_global_tracker_sync_mptcp_addrs(NMPGlobalTracker *self, gboolean reapply)
                 nm_assert_not_reached();
 
             if (i != j)
-                *(nm_g_array_index_p(entries, MptcpSyncData, j)) = *d;
+                (nm_g_array_index(entries, MptcpSyncData, j)) = *d;
             j++;
 
             if (j >= MPTCP_PM_ADDR_MAX) {
@@ -915,7 +931,7 @@ keep_and_next:
 
     if (entries) {
         for (i = 0; i < entries->len; i++) {
-            const MptcpSyncData       *d          = nm_g_array_index_p(entries, MptcpSyncData, i);
+            const MptcpSyncData       *d          = &nm_g_array_index(entries, MptcpSyncData, i);
             const NMPlatformMptcpAddr *mptcp_addr = NMP_OBJECT_CAST_MPTCP_ADDR(d->obj_data->obj);
             const NMPObject           *kobj;
 
@@ -1139,17 +1155,7 @@ nmp_global_tracker_track_rule_default(NMPGlobalTracker *self,
     /* track the default rules. See also `man ip-rule`. */
 
     if (NM_IN_SET(addr_family, AF_UNSPEC, AF_INET)) {
-        nmp_global_tracker_track_rule(self,
-                                      &((NMPlatformRoutingRule){
-                                          .addr_family = AF_INET,
-                                          .priority    = 0,
-                                          .table       = RT_TABLE_LOCAL,
-                                          .action      = FR_ACT_TO_TBL,
-                                          .protocol    = RTPROT_KERNEL,
-                                      }),
-                                      track_priority,
-                                      user_tag,
-                                      NULL);
+        nmp_global_tracker_track_local_rule(self, addr_family, track_priority, user_tag, NULL);
         nmp_global_tracker_track_rule(self,
                                       &((NMPlatformRoutingRule){
                                           .addr_family = AF_INET,
@@ -1174,17 +1180,7 @@ nmp_global_tracker_track_rule_default(NMPGlobalTracker *self,
                                       NULL);
     }
     if (NM_IN_SET(addr_family, AF_UNSPEC, AF_INET6)) {
-        nmp_global_tracker_track_rule(self,
-                                      &((NMPlatformRoutingRule){
-                                          .addr_family = AF_INET6,
-                                          .priority    = 0,
-                                          .table       = RT_TABLE_LOCAL,
-                                          .action      = FR_ACT_TO_TBL,
-                                          .protocol    = RTPROT_KERNEL,
-                                      }),
-                                      track_priority,
-                                      user_tag,
-                                      NULL);
+        nmp_global_tracker_track_local_rule(self, addr_family, track_priority, user_tag, NULL);
         nmp_global_tracker_track_rule(self,
                                       &((NMPlatformRoutingRule){
                                           .addr_family = AF_INET6,
@@ -1199,6 +1195,45 @@ nmp_global_tracker_track_rule_default(NMPGlobalTracker *self,
     }
 }
 
+void
+nmp_global_tracker_track_local_rule(NMPGlobalTracker *self,
+                                    int               addr_family,
+                                    gint32            track_priority,
+                                    gconstpointer     user_tag,
+                                    gconstpointer     user_tag_untrack)
+{
+    g_return_if_fail(NMP_IS_GLOBAL_TRACKER(self));
+
+    nm_assert(NM_IN_SET(addr_family, AF_UNSPEC, AF_INET, AF_INET6));
+
+    if (NM_IN_SET(addr_family, AF_UNSPEC, AF_INET)) {
+        nmp_global_tracker_track_rule(self,
+                                      &((NMPlatformRoutingRule){
+                                          .addr_family = AF_INET,
+                                          .priority    = 0,
+                                          .table       = RT_TABLE_LOCAL,
+                                          .action      = FR_ACT_TO_TBL,
+                                          .protocol    = RTPROT_KERNEL,
+                                      }),
+                                      track_priority,
+                                      user_tag,
+                                      user_tag_untrack);
+    }
+    if (NM_IN_SET(addr_family, AF_UNSPEC, AF_INET6)) {
+        nmp_global_tracker_track_rule(self,
+                                      &((NMPlatformRoutingRule){
+                                          .addr_family = AF_INET6,
+                                          .priority    = 0,
+                                          .table       = RT_TABLE_LOCAL,
+                                          .action      = FR_ACT_TO_TBL,
+                                          .protocol    = RTPROT_KERNEL,
+                                      }),
+                                      track_priority,
+                                      user_tag,
+                                      user_tag_untrack);
+    }
+}
+
 /*****************************************************************************/
 
 NMPGlobalTracker *
