@@ -16,6 +16,8 @@
 #include <readline/history.h>
 #include <readline/readline.h>
 #endif
+#include <gio/gunixinputstream.h>
+
 #include "libnm-client-aux-extern/nm-libnm-aux.h"
 
 #include "libnmc-base/nm-vpn-helpers.h"
@@ -352,15 +354,15 @@ print_ip_config(NMIPConfig      *cfg,
             g_strdup_printf("IP%c.%s", nm_utils_addr_family_to_char(addr_family), one_field);
     }
 
-    if (!nmc_print(nmc_config,
-                   (gpointer[]){cfg, NULL},
-                   NULL,
-                   NULL,
-                   addr_family == AF_INET
-                       ? NMC_META_GENERIC_GROUP("IP4", metagen_ip4_config, N_("GROUP"))
-                       : NMC_META_GENERIC_GROUP("IP6", metagen_ip6_config, N_("GROUP")),
-                   field_str,
-                   &error)) {
+    if (!nmc_print_table(nmc_config,
+                         (gpointer[]){cfg, NULL},
+                         NULL,
+                         NULL,
+                         addr_family == AF_INET
+                             ? NMC_META_GENERIC_GROUP("IP4", metagen_ip4_config, N_("GROUP"))
+                             : NMC_META_GENERIC_GROUP("IP6", metagen_ip6_config, N_("GROUP")),
+                         field_str,
+                         &error)) {
         return FALSE;
     }
     return TRUE;
@@ -383,15 +385,15 @@ print_dhcp_config(NMDhcpConfig    *dhcp,
             g_strdup_printf("DHCP%c.%s", nm_utils_addr_family_to_char(addr_family), one_field);
     }
 
-    if (!nmc_print(nmc_config,
-                   (gpointer[]){dhcp, NULL},
-                   NULL,
-                   NULL,
-                   addr_family == AF_INET
-                       ? NMC_META_GENERIC_GROUP("DHCP4", metagen_dhcp_config, N_("GROUP"))
-                       : NMC_META_GENERIC_GROUP("DHCP6", metagen_dhcp_config, N_("GROUP")),
-                   field_str,
-                   &error)) {
+    if (!nmc_print_table(nmc_config,
+                         (gpointer[]){dhcp, NULL},
+                         NULL,
+                         NULL,
+                         addr_family == AF_INET
+                             ? NMC_META_GENERIC_GROUP("DHCP4", metagen_dhcp_config, N_("GROUP"))
+                             : NMC_META_GENERIC_GROUP("DHCP6", metagen_dhcp_config, N_("GROUP")),
+                         field_str,
+                         &error)) {
         return FALSE;
     }
     return TRUE;
@@ -430,10 +432,13 @@ nmc_find_connection(const GPtrArray *connections,
     GPtrArray                   *result              = out_result ? *out_result : NULL;
     const guint                  result_inital_len   = result ? result->len : 0u;
     guint                        i, j;
+    gboolean                     must_match_uniquely;
 
     nm_assert(connections);
     nm_assert(filter_val);
 
+    must_match_uniquely = NM_IN_STRSET(filter_type, "uuid", "path");
+
     for (i = 0; i < connections->len; i++) {
         gboolean      match_by_uuid = FALSE;
         NMConnection *connection;
@@ -450,6 +455,12 @@ nmc_find_connection(const GPtrArray *connections,
                 match_by_uuid = TRUE;
                 goto found;
             }
+            if (filter_type && !nm_str_is_empty(filter_val) && g_str_has_prefix(v, filter_val)) {
+                /* If the selector is qualified by "uuid", prefix matches for the UUID are
+                 * also OK. At least, if they result in a unique match. */
+                nm_assert(must_match_uniquely);
+                goto found;
+            }
         }
 
         if (NM_IN_STRSET(filter_type, NULL, "id")) {
@@ -469,7 +480,8 @@ nmc_find_connection(const GPtrArray *connections,
                 goto found;
         }
 
-        if (NM_IN_STRSET(filter_type, NULL, "filename")) {
+        if (NM_IS_REMOTE_CONNECTION(connections->pdata[i])
+            && NM_IN_STRSET(filter_type, NULL, "filename")) {
             v = nm_remote_connection_get_filename(NM_REMOTE_CONNECTION(connections->pdata[i]));
             if (complete && (filter_type || *filter_val))
                 nmc_complete_strings(filter_val, v);
@@ -480,14 +492,26 @@ nmc_find_connection(const GPtrArray *connections,
         continue;
 
 found:
+
+        if (must_match_uniquely && (best_candidate || best_candidate_uuid)) {
+            /* We found duplicates. This is wrong. */
+            if (out_result && *out_result) {
+                /* Remove the element that we added before. */
+                g_ptr_array_set_size(*out_result, result_inital_len);
+            }
+            return NULL;
+        }
+
         if (match_by_uuid) {
             if (!complete && !out_result)
                 return connection;
-            best_candidate_uuid = connection;
+            if (!best_candidate_uuid)
+                best_candidate_uuid = connection;
         } else {
             if (!best_candidate)
                 best_candidate = connection;
         }
+
         if (out_result) {
             gboolean already_tracked = FALSE;
 
@@ -536,8 +560,6 @@ nmc_find_active_connection(const GPtrArray *active_cons,
         NMActiveConnection *candidate = g_ptr_array_index(active_cons, i);
         const char         *v, *v_num;
 
-        con = nm_active_connection_get_connection(candidate);
-
         /* When filter_type is NULL, compare connection ID (filter_val)
          * against all types. Otherwise, only compare against the specific
          * type. If 'path' or 'apath' filter types are specified, comparison
@@ -559,6 +581,8 @@ nmc_find_active_connection(const GPtrArray *active_cons,
                 goto found;
         }
 
+        con = nm_active_connection_get_connection(candidate);
+
         if (NM_IN_STRSET(filter_type, NULL, "path")) {
             v     = con ? nm_connection_get_path(NM_CONNECTION(con)) : NULL;
             v_num = nm_utils_dbus_path_get_last_component(v);
@@ -569,7 +593,7 @@ nmc_find_active_connection(const GPtrArray *active_cons,
         }
 
         if (NM_IN_STRSET(filter_type, NULL, "filename")) {
-            v = nm_remote_connection_get_filename(con);
+            v = con ? nm_remote_connection_get_filename(con) : NULL;
             if (complete && (filter_type || *filter_val))
                 nmc_complete_strings(filter_val, v);
             if (nm_streq0(filter_val, v))
@@ -636,16 +660,16 @@ vpn_openconnect_get_secrets(NMConnection *connection, GPtrArray *secrets)
     /* Interactively authenticate to OpenConnect server and get secrets */
     ret = nm_vpn_openconnect_authenticate_helper(gw, &cookie, &gateway, &gwcert, &status, &error);
     if (!ret) {
-        g_printerr(_("Error: openconnect failed: %s\n"), error->message);
+        nmc_printerr(_("Error: openconnect failed: %s\n"), error->message);
         g_clear_error(&error);
         return FALSE;
     }
 
     if (WIFEXITED(status)) {
         if (WEXITSTATUS(status) != 0)
-            g_printerr(_("Error: openconnect failed with status %d\n"), WEXITSTATUS(status));
+            nmc_printerr(_("Error: openconnect failed with status %d\n"), WEXITSTATUS(status));
     } else if (WIFSIGNALED(status))
-        g_printerr(_("Error: openconnect failed with signal %d\n"), WTERMSIG(status));
+        nmc_printerr(_("Error: openconnect failed with signal %d\n"), WTERMSIG(status));
 
     /* Append port to the host value */
     if (gateway && port) {
@@ -719,7 +743,7 @@ get_secrets_from_user(const NmcConfig *nmc_config,
                     }
                 }
                 if (msg)
-                    g_print("%s\n", msg);
+                    nmc_print("%s\n", msg);
 
                 echo_on = secret->is_secret ? nmc_config->show_secrets : TRUE;
 
@@ -736,10 +760,10 @@ get_secrets_from_user(const NmcConfig *nmc_config,
                     pwd = g_strdup("");
             } else {
                 if (msg)
-                    g_print("%s\n", msg);
-                g_printerr(_("Warning: password for '%s' not given in 'passwd-file' "
-                             "and nmcli cannot ask without '--ask' option.\n"),
-                           secret->entry_id);
+                    nmc_print("%s\n", msg);
+                nmc_printerr(_("Warning: password for '%s' not given in 'passwd-file' "
+                               "and nmcli cannot ask without '--ask' option.\n"),
+                             secret->entry_id);
             }
         }
         /* No password provided, cancel the secrets. */
@@ -871,7 +895,10 @@ static void
 readline_cb(char *line)
 {
     rl_got_line = TRUE;
-    rl_string   = line;
+
+    free(rl_string);
+    rl_string = line;
+
     rl_callback_handler_remove();
 }
 
@@ -886,19 +913,15 @@ static char *
 nmc_readline_helper(const NmcConfig *nmc_config, const char *prompt)
 {
     GSource *io_source;
+    char    *result;
 
     nmc_set_in_readline(TRUE);
 
-    io_source = nm_g_unix_fd_source_new(STDIN_FILENO,
-                                        G_IO_IN,
-                                        G_PRIORITY_DEFAULT,
-                                        stdin_ready_cb,
-                                        NULL,
-                                        NULL);
-    g_source_attach(io_source, NULL);
+    io_source = nm_g_unix_fd_add_source(STDIN_FILENO, G_IO_IN, stdin_ready_cb, NULL);
 
 read_again:
-    rl_string   = NULL;
+    nm_clear_free(&rl_string);
+
     rl_got_line = FALSE;
     rl_callback_handler_install(prompt, readline_cb);
 
@@ -924,7 +947,6 @@ read_again:
         if (nmc_config->in_editor || (rl_string && *rl_string)) {
             /* In editor, or the line is not empty */
             /* Call readline again to get new prompt (repeat) */
-            g_free(rl_string);
             goto read_again;
         } else {
             /* Not in editor and line is empty, exit */
@@ -932,20 +954,24 @@ read_again:
         }
     } else if (!rl_string) {
         /* Ctrl-D, exit */
-        nmc_exit();
+        if (g_main_loop_is_running(loop))
+            nmc_exit();
     }
 
     /* Return NULL, not empty string */
-    if (rl_string && *rl_string == '\0') {
-        g_free(rl_string);
-        rl_string = NULL;
-    }
+    if (rl_string && *rl_string == '\0')
+        nm_clear_free(&rl_string);
 
     nm_clear_g_source_inst(&io_source);
 
     nmc_set_in_readline(FALSE);
 
-    return rl_string;
+    if (!rl_string)
+        return NULL;
+
+    result = g_strdup(rl_string);
+    nm_clear_free(&rl_string);
+    return result;
 }
 
 /**
@@ -1279,12 +1305,157 @@ got_client(GObject *source_object, GAsyncResult *res, gpointer user_data)
     nm_g_slice_free(call);
 }
 
+typedef struct {
+    GString *str;
+    char     buf[512];
+    CmdCall *call;
+} CmdStdinData;
+
+static void read_offline_connection_next(GInputStream *stream, CmdStdinData *data);
+
+static void
+read_offline_connection_chunk(GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+    GInputStream                   *stream   = G_INPUT_STREAM(source_object);
+    CmdStdinData                   *data     = user_data;
+    CmdCall                        *call     = data->call;
+    gs_unref_object GTask          *task     = NULL;
+    nm_auto_unref_keyfile GKeyFile *keyfile  = NULL;
+    gs_free char                   *base_dir = NULL;
+    GError                         *error    = NULL;
+    gssize                          bytes_read;
+    NMConnection                   *connection;
+    NmCli                          *nmc;
+
+    bytes_read = g_input_stream_read_finish(stream, res, &error);
+    if (bytes_read > 0) {
+        /* We need to read more. */
+        g_string_append_len(data->str, data->buf, bytes_read);
+        read_offline_connection_next(stream, data);
+        return;
+    }
+
+    /* End reached. */
+
+    task = g_steal_pointer(&call->task);
+    nmc  = g_task_get_task_data(task);
+    nmc->should_wait--;
+
+    if (bytes_read == -1) {
+        g_task_return_error(task, error);
+        goto finish;
+    }
+
+    keyfile = g_key_file_new();
+    if (!g_key_file_load_from_data(keyfile,
+                                   data->str->str,
+                                   data->str->len,
+                                   G_KEY_FILE_NONE,
+                                   &error)) {
+        g_task_return_error(task, error);
+        goto finish;
+    }
+
+    base_dir = g_get_current_dir();
+    connection =
+        nm_keyfile_read(keyfile, base_dir, NM_KEYFILE_HANDLER_FLAGS_NONE, NULL, NULL, &error);
+    if (!connection) {
+        g_task_return_error(task, error);
+        goto finish;
+    }
+
+    g_ptr_array_add(nmc->offline_connections, connection);
+    call->cmd->func(call->cmd, nmc, call->argc, (const char *const *) call->argv);
+    g_task_return_boolean(task, TRUE);
+
+finish:
+    g_strfreev(call->argv);
+    nm_g_slice_free(call);
+    g_string_free(data->str, TRUE);
+    nm_g_slice_free(data);
+}
+
+static void
+read_offline_connection_next(GInputStream *stream, CmdStdinData *data)
+{
+    g_input_stream_read_async(stream,
+                              data->buf,
+                              sizeof(data->buf),
+                              G_PRIORITY_DEFAULT,
+                              NULL,
+                              read_offline_connection_chunk,
+                              data);
+}
+
+static void
+read_offline_connection(CmdCall *call)
+{
+    gs_unref_object GInputStream *stream = NULL;
+    CmdStdinData                 *data;
+
+    stream     = g_unix_input_stream_new(STDIN_FILENO, TRUE);
+    data       = g_slice_new(CmdStdinData);
+    data->call = call;
+    data->str  = g_string_new_len(NULL, sizeof(data->buf));
+
+    read_offline_connection_next(stream, data);
+}
+
+static NMConnection *
+dummy_offline_connection(void)
+{
+    NMConnection *connection;
+
+    connection = nm_simple_connection_new();
+    nm_connection_add_setting(connection, nm_setting_connection_new());
+    return connection;
+}
+
 static void
 call_cmd(NmCli *nmc, GTask *task, const NMCCommand *cmd, int argc, const char *const *argv)
 {
     CmdCall *call;
 
-    if (nmc->client || !cmd->needs_client) {
+    if (nmc->offline) {
+        if (!cmd->supports_offline) {
+            g_task_return_new_error(task,
+                                    NMCLI_ERROR,
+                                    NMC_RESULT_ERROR_USER_INPUT,
+                                    _("Error: command doesn't support --offline mode."));
+            g_object_unref(task);
+            return;
+        }
+
+        if (!nmc->offline_connections)
+            nmc->offline_connections = g_ptr_array_new_full(1, g_object_unref);
+
+        if (cmd->needs_offline_conn) {
+            g_return_if_fail(nmc->offline_connections->len == 0);
+
+            if (nmc->complete) {
+                g_ptr_array_add(nmc->offline_connections, dummy_offline_connection());
+                cmd->func(cmd, nmc, argc, argv);
+                g_task_return_boolean(task, TRUE);
+                g_object_unref(task);
+                return;
+            }
+
+            nmc->should_wait++;
+            call  = g_slice_new(CmdCall);
+            *call = (CmdCall){
+                .cmd  = cmd,
+                .argc = argc,
+                .argv = nm_strv_dup(argv, argc, TRUE),
+                .task = task,
+            };
+            read_offline_connection(call);
+            return;
+        } else {
+            cmd->func(cmd, nmc, argc, argv);
+            g_task_return_boolean(task, TRUE);
+            g_object_unref(task);
+        }
+    } else if (nmc->client || !cmd->needs_client) {
         /* Check whether NetworkManager is running */
         if (cmd->needs_nm_running && !nm_client_get_nm_running(nmc->client)) {
             g_task_return_new_error(task,
@@ -1361,7 +1532,7 @@ nmc_do_cmd(NmCli *nmc, const NMCCommand cmds[], const char *cmd, int argc, const
     if (argc == 1 && nmc->complete) {
         for (c = cmds; c->cmd; ++c) {
             if (!*cmd || matches(cmd, c->cmd))
-                g_print("%s\n", c->cmd);
+                nmc_print("%s\n", c->cmd);
         }
         nmc_complete_help(cmd);
         g_task_return_boolean(task, TRUE);
@@ -1441,7 +1612,7 @@ nmc_complete_strv(const char *prefix, gssize nargs, const char *const *args)
         if (prefix && !matches(prefix, candidate))
             continue;
 
-        g_print("%s\n", candidate);
+        nmc_print("%s\n", candidate);
     }
 }
 
