@@ -11,7 +11,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
-#include <termios.h>
 #include <unistd.h>
 #include <locale.h>
 #if HAVE_EDITLINE_READLINE
@@ -44,6 +43,7 @@
             [NM_META_COLOR_CONNECTION_DISCONNECTING] = "31",   \
             [NM_META_COLOR_CONNECTION_INVISIBLE]     = "2",    \
             [NM_META_COLOR_CONNECTION_EXTERNAL]      = "32;2", \
+            [NM_META_COLOR_CONNECTION_DEPRECATED]    = "2",    \
             [NM_META_COLOR_CONNECTIVITY_FULL]        = "32",   \
             [NM_META_COLOR_CONNECTIVITY_LIMITED]     = "33",   \
             [NM_META_COLOR_CONNECTIVITY_NONE]        = "31",   \
@@ -74,6 +74,7 @@
             [NM_META_COLOR_WIFI_SIGNAL_GOOD]         = "33",   \
             [NM_META_COLOR_WIFI_SIGNAL_POOR]         = "36",   \
             [NM_META_COLOR_WIFI_SIGNAL_UNKNOWN]      = "2",    \
+            [NM_META_COLOR_WIFI_DEPRECATED]          = "2",    \
             [NM_META_COLOR_ENABLED]                  = "32",   \
             [NM_META_COLOR_DISABLED]                 = "31",   \
         },                                                     \
@@ -118,8 +119,7 @@ typedef struct {
 } ArgsInfo;
 
 /* --- Global variables --- */
-GMainLoop     *loop = NULL;
-struct termios termios_orig;
+GMainLoop *loop = NULL;
 
 NM_CACHED_QUARK_FCN("nmcli-error-quark", nmcli_error_quark);
 
@@ -166,13 +166,13 @@ complete_one(gpointer key, gpointer value, gpointer user_data)
             /* value prefix was not a standalone argument,
              * it was part of --option=<value> argument.
              * Repeat the part leading to "=". */
-            g_print("%s=", option);
+            nmc_print("%s=", option);
         }
-        g_print("%.*s%s%s\n",
-                (int) (last - prefix),
-                prefix,
-                name,
-                strcmp(last, name) == 0 ? "," : "");
+        nmc_print("%.*s%s%s\n",
+                  (int) (last - prefix),
+                  prefix,
+                  name,
+                  strcmp(last, name) == 0 ? "," : "");
     }
 }
 
@@ -228,9 +228,9 @@ complete_option_with_value(const char *option, const char *prefix, ...)
                 /* value prefix was not a standalone argument,
                  * it was part of --option=<value> argument.
                  * Repeat the part leading to "=". */
-                g_print("%s=", option);
+                nmc_print("%s=", option);
             }
-            g_print("%s\n", candidate);
+            nmc_print("%s\n", candidate);
         }
     }
     va_end(args);
@@ -239,7 +239,7 @@ complete_option_with_value(const char *option, const char *prefix, ...)
 static void
 usage(void)
 {
-    g_printerr(_(
+    nmc_printerr(_(
         "Usage: nmcli [OPTIONS] OBJECT { COMMAND | help }\n"
         "\n"
         "OPTIONS\n"
@@ -551,6 +551,7 @@ static NM_UTILS_STRING_TABLE_LOOKUP_DEFINE(
     {"connection-external", NM_META_COLOR_CONNECTION_EXTERNAL},
     {"connection-invisible", NM_META_COLOR_CONNECTION_INVISIBLE},
     {"connection-unknown", NM_META_COLOR_CONNECTION_UNKNOWN},
+    {"connection-deprecated", NM_META_COLOR_CONNECTION_DEPRECATED},
     {"connectivity-full", NM_META_COLOR_CONNECTIVITY_FULL},
     {"connectivity-limited", NM_META_COLOR_CONNECTIVITY_LIMITED},
     {"connectivity-none", NM_META_COLOR_CONNECTIVITY_NONE},
@@ -587,7 +588,8 @@ static NM_UTILS_STRING_TABLE_LOOKUP_DEFINE(
     {"wifi-signal-fair", NM_META_COLOR_WIFI_SIGNAL_FAIR},
     {"wifi-signal-good", NM_META_COLOR_WIFI_SIGNAL_GOOD},
     {"wifi-signal-poor", NM_META_COLOR_WIFI_SIGNAL_POOR},
-    {"wifi-signal-unknown", NM_META_COLOR_WIFI_SIGNAL_UNKNOWN}, );
+    {"wifi-signal-unknown", NM_META_COLOR_WIFI_SIGNAL_UNKNOWN},
+    {"wifi-deprecated", NM_META_COLOR_WIFI_DEPRECATED}, );
 
 static gboolean
 parse_color_scheme(char *palette_buffer, NmcColorPalette *out_palette, GError **error)
@@ -724,7 +726,7 @@ process_command_line(NmCli *nmc, int argc, char **argv_orig)
         {"monitor", nmc_command_func_monitor, NULL, TRUE, FALSE},
         {"networking", nmc_command_func_networking, NULL, FALSE, FALSE},
         {"radio", nmc_command_func_radio, NULL, FALSE, FALSE},
-        {"connection", nmc_command_func_connection, NULL, FALSE, FALSE},
+        {"connection", nmc_command_func_connection, NULL, FALSE, FALSE, TRUE},
         {"device", nmc_command_func_device, NULL, FALSE, FALSE},
         {"agent", nmc_command_func_agent, NULL, FALSE, FALSE},
         {NULL, nmc_command_func_overview, usage, TRUE, TRUE},
@@ -759,15 +761,16 @@ process_command_line(NmCli *nmc, int argc, char **argv_orig)
 
         if (argc == 1 && nmc->complete) {
             nmc_complete_strings(argv[0],
+                                 "--overview",
+                                 "--offline",
                                  "--terse",
                                  "--pretty",
                                  "--mode",
-                                 "--overview",
                                  "--colors",
                                  "--escape",
                                  "--fields",
-                                 "--nocheck",
                                  "--get-values",
+                                 "--nocheck",
                                  "--wait",
                                  "--version",
                                  "--help");
@@ -781,6 +784,8 @@ process_command_line(NmCli *nmc, int argc, char **argv_orig)
 
         if (matches_arg(nmc, &argc, &argv, "-overview", NULL)) {
             nmc->nmc_config_mutable.overview = TRUE;
+        } else if (matches_arg(nmc, &argc, &argv, "-offline", NULL)) {
+            nmc->offline = TRUE;
         } else if (matches_arg(nmc, &argc, &argv, "-terse", NULL)) {
             if (nmc->nmc_config.print_output == NMC_PRINT_TERSE) {
                 g_string_printf(nmc->return_text,
@@ -884,7 +889,7 @@ process_command_line(NmCli *nmc, int argc, char **argv_orig)
             nmc->timeout = (int) timeout;
         } else if (matches_arg(nmc, &argc, &argv, "-version", NULL)) {
             if (!nmc->complete)
-                g_print(_("nmcli tool, version %s\n"), NMCLI_VERSION);
+                nmc_print(_("nmcli tool, version %s\n"), NMCLI_VERSION);
             return NMC_RESULT_SUCCESS;
         } else if (matches_arg(nmc, &argc, &argv, "-help", NULL)) {
             if (!nmc->complete)
@@ -935,7 +940,6 @@ nmc_clear_sigint(void)
 void
 nmc_exit(void)
 {
-    tcsetattr(STDIN_FILENO, TCSADRAIN, &termios_orig);
     nmc_cleanup_readline();
     exit(1);
 }
@@ -1010,6 +1014,8 @@ nmc_cleanup(NmCli *nmc)
 
     nm_clear_g_free(&nmc->palette_buffer);
 
+    nm_clear_pointer(&nmc->offline_connections, g_ptr_array_unref);
+
     nmc_polkit_agent_fini(nmc);
 }
 
@@ -1026,9 +1032,6 @@ main(int argc, char *argv[])
     textdomain(GETTEXT_PACKAGE);
 #endif
 
-    /* Save terminal settings */
-    tcgetattr(STDIN_FILENO, &termios_orig);
-
     nm_cli.return_text = g_string_new(_("Success"));
     loop               = g_main_loop_new(NULL, FALSE);
 
@@ -1044,7 +1047,7 @@ main(int argc, char *argv[])
             nm_cli.return_value = NMC_RESULT_SUCCESS;
     } else if (nm_cli.return_value != NMC_RESULT_SUCCESS) {
         /* Print result descripting text */
-        g_printerr("%s\n", nm_cli.return_text->str);
+        nmc_printerr("%s\n", nm_cli.return_text->str);
     }
 
     nmc_cleanup(&nm_cli);
