@@ -23,15 +23,15 @@ same_lines() {
 
 libnm_headers() {
     (
-        ls -1 "$1/libnm"/*.h "$1/libnm-core"/*.h | \
+        ls -1 "$1/src/libnm-client-public"/*.h "$1/src/libnm-core-public"/*.h | \
         if [ -n "$2" ]; then
-            grep -v -F "$1/libnm-core/nm-core-enum-types.h" | \
-            grep -v -F "$1/libnm/nm-enum-types.h"
+            grep -v -F "$1/src/libnm-core-public/nm-core-enum-types.h" | \
+            grep -v -F "$1/src/libnm-client-public/nm-enum-types.h"
         else
             cat
         fi
         if [ -n "$2" ]; then
-            ls -1 "$2/libnm"/*.h "$2/libnm-core"/*.h
+            ls -1 "$2/src/libnm-client-public"/*.h "$2/src/libnm-core-public"/*.h
         fi
     ) | sort | uniq
 }
@@ -63,18 +63,42 @@ F1_EXTRA="
 nm-core-enum-types
 nm-enum-types
 "
-F2="$(grep -l "$(sed -n 's/^[\t ]*\(.*_get_type\);/\1/p' "$SOURCEDIR/libnm/libnm.ver" | word_regex)" \
+F2="$(grep -l "$(sed -n 's/^[\t ]*\(.*_get_type\);/\1/p' "$SOURCEDIR/src/libnm-client-impl/libnm.ver" | word_regex)" \
            $(libnm_headers "$SOURCEDIR" "$BUILDDIR") \
       | sed 's,.*/\([^/]\+\)\.h$,\1,')"
 F2_EXTRA="
 annotation-glossary
 api-index-full
+nm-conn-utils
 nm-dbus-interface
 nm-errors
+nm-ethtool-utils
 nm-keyfile
 nm-utils
 nm-version
+nm-version-macros
+nm-vpn-dbus-interface
 "
 if ! same_lines "$F1"$'\n'"$F1_EXTRA" "$F2"$'\n'"$F2_EXTRA"; then
     die "*** Error: libnm classes not included in docs/libnm/libnm-docs.xml ***"
 fi
+
+F1="$(sed -n 's/^#include "\(nm-.*\).h"$/xml\/\1.xml/p' "$SOURCEDIR/src/libnm-client-public/NetworkManager.h")"
+F1_EXTRA="
+xml/annotation-glossary.xml
+xml/api-index-full.xml
+xml/nm-setting-ovs-external-ids.xml
+xml/nm-setting-ovs-other-config.xml
+xml/nm-version-macros.xml
+xml/nm-secret-agent-old.xml
+xml/nm-vpn-plugin-old.xml
+"
+F2="$(sed -n 's/.*<xi:include href="\(xml\/.*.xml\)".*/\1/p' "$SOURCEDIR/docs/libnm/libnm-docs.xml")"
+F2_EXTRA="
+xml/nm-autoptr.xml
+xml/nm-core-enum-types.xml
+xml/nm-enum-types.xml
+"
+if ! same_lines "$F1"$'\n'"$F1_EXTRA" "$F2"$'\n'"$F2_EXTRA"; then
+    die "*** Error: documentation from public headers not included in docs/libnm/libnm-docs.xml ***"
+fi
